@@ -1,4 +1,4 @@
-use crate::intern::PreprocessedMft;
+use crate::intern::{PreprocessedMft};
 use crate::{FilenameInfo, TimestampTuple};
 use anyhow::Result;
 use likely_stable::unlikely;
@@ -14,7 +14,7 @@ use usnjrnl::{CommonUsnRecord, UsnRecordData};
 /// a bodyfile line, which would be
 ///
 ///  - the base reference (needed to print the `inode` number)
-///  - the `$FILENAME` attribute. One file can have more than one `$FILENAME`
+///  - the `$FILE_NAME` attribute. One file can have more than one `$FILE_NAME`
 ///    attribbutes, but we store only one of them. We choose the right attribute
 ///    using the following priority:
 ///    
@@ -22,7 +22,7 @@ use usnjrnl::{CommonUsnRecord, UsnRecordData};
 ///    2. `Win32`
 ///    3. `POSIX`
 ///    4. ´DOS`
-///    If a file doesn't have a `$FILENAME` attribute, which may happen with already deleted files,
+///    If a file doesn't have a `$FILE_NAME` attribute, which may happen with already deleted files,
 ///    then a filename is being generated, but *not* stored in `file_name_attribute`
 ///
 ///    This attribute is required to display the filename, but also contains four timestamps,
@@ -35,7 +35,8 @@ pub struct CompleteMftEntry {
     standard_info_timestamps: Option<TimestampTuple>,
     full_path: RefCell<String>,
     is_allocated: bool,
-    usnjrnl_records: Vec<CommonUsnRecord>
+    deletion_status: RefCell<&'static str>,
+    usnjrnl_records: Vec<CommonUsnRecord>,
 }
 
 impl CompleteMftEntry {
@@ -47,6 +48,7 @@ impl CompleteMftEntry {
             full_path: RefCell::new(String::new()),
             is_allocated: entry.is_allocated(),
             usnjrnl_records: Vec::new(),
+            deletion_status: RefCell::new(if entry.is_allocated() {""} else {" (deleted)"} )
         };
         c.update_attributes(
             &entry,
@@ -66,6 +68,7 @@ impl CompleteMftEntry {
             full_path: RefCell::new(String::new()),
             is_allocated: false,
             usnjrnl_records: Vec::new(),
+            deletion_status: RefCell::new(" (deleted)")
         };
         c.add_nonbase_entry(entry);
         c
@@ -83,6 +86,7 @@ impl CompleteMftEntry {
             full_path: RefCell::new(String::new()),
             is_allocated: false,
             usnjrnl_records: records,
+            deletion_status: RefCell::new(" (deleted)")
         }
     }
 
@@ -90,6 +94,10 @@ impl CompleteMftEntry {
         &self.base_entry
     }
 
+    pub fn is_allocated(&self) -> bool {
+        self.is_allocated
+    }
+
     pub fn set_base_entry(&mut self, entry_ref: MftReference, entry: MftEntry) {
         assert_eq!(self.base_entry, entry_ref);
 
@@ -111,7 +119,7 @@ impl CompleteMftEntry {
         records.sort_by(
             |a, b| a.data.timestamp().partial_cmp(b.data.timestamp()).unwrap());
 
-        if self.usnjrnl_records.len() == 0 {
+        if self.usnjrnl_records.is_empty() {
             self.usnjrnl_records = records;
         } else {
             self.usnjrnl_records.extend(records);
@@ -176,12 +184,25 @@ impl CompleteMftEntry {
         }
     }
 
+    fn set_folder_name(&self, mft: &PreprocessedMft, parent: &MftReference, my_name: &str) {
+        assert_ne!(parent, &self.base_entry);
+        let mut fp = self.full_path.borrow_mut();
+
+        let parent_info = mft.get_full_path(parent);
+
+        *fp = parent_info.full_path;
+        if ! &fp.ends_with('/') {
+            fp.push('/');
+        }
+        fp.push_str(my_name);
+    }
+
     pub fn get_full_path(&self, mft: &PreprocessedMft) -> String {
         if unlikely(self.full_path.borrow().is_empty()) {
             if self.base_entry.entry == 5
             /* matchs the root entry */
             {
-                *self.full_path.borrow_mut() = String::from("");
+                *self.full_path.borrow_mut() = String::from("/");
                 return self.full_path.borrow().clone();
             }
 
@@ -189,16 +210,7 @@ impl CompleteMftEntry {
                 Some(name) => {
                     match self.parent() {
                         None => *self.full_path.borrow_mut() = name.filename().clone(),
-                        Some(p) => {
-                            // prevent endless recursion, mainly for $MFT entry 5 (which is the root directory)
-                            assert_ne!(p, &self.base_entry);
-
-                            let parent_path = mft.get_full_path(p);
-                            let mut fp = self.full_path.borrow_mut();
-                            *fp = parent_path;
-                            fp.push('/');
-                            fp.push_str(name.filename());
-                        }
+                        Some(p) => self.set_folder_name(mft, p, name.filename())
                     }
                 }
                 None => {
@@ -212,13 +224,7 @@ impl CompleteMftEntry {
 
                     match self.parent_from_usnjrnl() {
                         None => *self.full_path.borrow_mut() = my_name,
-                        Some(parent) => {
-                            let parent_path = mft.get_full_path(&parent);
-                            let mut fp = self.full_path.borrow_mut();
-                            *fp = parent_path;
-                            fp.push('/');
-                            fp.push_str(&my_name);
-                        }
+                        Some(p) => self.set_folder_name(mft, &p, &my_name)
                     };
                 }
             }
@@ -227,7 +233,7 @@ impl CompleteMftEntry {
     }
 
     fn filename_from_usnjrnl(&self) -> Option<&str> {
-        self.usnjrnl_records.last().and_then(|r| Some(r.data.filename()))
+        self.usnjrnl_records.last().map(|r| r.data.filename())
     }
     
     fn parent_from_usnjrnl(&self) -> Option<MftReference> {
@@ -257,7 +263,7 @@ impl CompleteMftEntry {
         let mode = String::from("0");
         let uid = String::from("0");
         let gid = String::from("0");
-        let status = if self.is_allocated { "" } else { " (deleted)" };
+        let status = self.deletion_status.borrow();
         let filesize = self.filesize();
         format!(
             "0|{}{}|{}|{}|{}|{}|{}|{}|{}|{}|{}\n",
@@ -278,7 +284,7 @@ impl CompleteMftEntry {
     fn format_fn(&self, mft: &PreprocessedMft) -> Option<String> {
         match self.file_name_attribute {
             Some(ref fn_attr) => {
-                let display_name = format!("{} ($FILENAME)", self.get_full_path(mft));
+                let display_name = format!("{} ($FILE_NAME)", self.get_full_path(mft));
                 Some(self.format(
                     &display_name,
                     fn_attr.timestamps().accessed(),
@@ -292,16 +298,12 @@ impl CompleteMftEntry {
     }
 
     fn format_si(&self, mft: &PreprocessedMft) -> Option<String> {
-        match self.standard_info_timestamps {
-            Some(ref standard_info_timestamps) => Some(self.format(
-                &self.get_full_path(mft),
-                standard_info_timestamps.accessed(),
-                standard_info_timestamps.mft_modified(),
-                standard_info_timestamps.modified(),
-                standard_info_timestamps.created()),
-            ),
-            None => None,
-        }
+        self.standard_info_timestamps.as_ref().map(|si| self.format(
+            &self.get_full_path(mft),
+            si.accessed(),
+            si.mft_modified(),
+            si.modified(),
+            si.created()))
     }
 
     /// returns the filename stored in the `$MFT`, if any, or None
@@ -330,15 +332,26 @@ impl CompleteMftEntry {
                     format!(" reason={}", data.Reason)
                 };
 
-                let parent_info = match &self.file_name_attribute {
-                    Some(fni) => {
-                        if fni.parent() != &data.ParentFileReferenceNumber {
-                            format!(" parent='{}'", mft.get_full_path(&data.ParentFileReferenceNumber))
-                        } else {
+                let parent_info = mft.get_full_path(&data.ParentFileReferenceNumber);
+                let parent_info = match &parent_info.reference {
+                    Some(parent_ref) => {
+                        if parent_ref == &data.ParentFileReferenceNumber {
                             "".to_owned()
+                        } else if !parent_info.is_allocated &&
+                            parent_ref == &MftReference::new(
+                                data.ParentFileReferenceNumber.entry,
+                                data.ParentFileReferenceNumber.sequence+1) {
+                            "".to_owned()
+                        } else {
+                            format!(" parent={}-{}/{}-{}/'{}'",
+                                parent_ref.entry,
+                                parent_ref.sequence,
+                                data.ParentFileReferenceNumber.entry,
+                                data.ParentFileReferenceNumber.sequence,
+                                parent_info.full_path)
                         }
                     }
-                    None => format!(" parent='{}'", mft.get_full_path(&data.ParentFileReferenceNumber))
+                    None => format!(" parent='{}'", parent_info.full_path)
                 };
 
                 let display_name = format!("{} ($UsnJrnl{}{}{})",
@@ -389,7 +402,7 @@ impl CompleteMftEntry {
     }
 
     pub fn bodyfile_lines_count(&self) -> usize {
-        return match &self.standard_info_timestamps {
+        (match &self.standard_info_timestamps {
             Some(_) => 1,
             None    => 0,
         }
@@ -399,7 +412,7 @@ impl CompleteMftEntry {
             None    => 0,
         }
         +
-        self.usnjrnl_records.len();
+        self.usnjrnl_records.len())
     }
 }
 
